import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/eyflutter_uikit.dart';
import 'package:flutter/material.dart';

mixin DropdownListController {
  void notifierLazy(String value);

  /// 设置组件启用状态
  void enable(bool enable);
}

/// DropdownList(
///   items: [
///     ListItem(value: "1", text: '选项1'),
///     ListItem(text: "选项2", value: "2"),
///   ],
///   width: 200,
///   title: "标题",
///   submitText: "确定",
///   isPreSelect: true,
///   isMultiSelect: true,
///   hintText: "请选择 ",
///   leastSelectHintText: "至少选择一项",
///   buttonBackgroundColors: const [Color(0xff1afa29), Color(0xff1afa29)],
///   buttonIconColor: Colors.white,
///   selectCall: (List<ListItem> selectItems) {},
/// )
class DropdownList extends StatefulWidget {
  /// 组件宽度
  final double? width;

  /// 组件高度
  final double height;

  /// 边框颜色
  final Color borderColor;

  /// 按钮背景颜色
  final List<Color> buttonBackgroundColors;

  /// 按钮禁用状态背景颜色
  final List<Color> buttonDisableBackgroundColors;

  /// 按钮图标颜色
  final Color buttonIconColor;

  /// 菜单项
  final List<ListItem> items;

  /// 选择组件显示标题
  final String? title;

  /// 提交按钮
  final Widget? submit;

  /// 提交按钮文本与submit互斥
  final String? submitText;

  /// true-选择后需要由确认完成;false-选择直接完成;
  final bool isPreSelect;

  /// 仅在isPreSelect=true时有效
  /// true-多选;false-单选;
  final bool isMultiSelect;

  /// 选择回调
  /// [selectItems] 选择项
  final Function(List<ListItem> selectItems) selectCall;

  /// 提示文本
  final String hintText;

  /// 至少选择提示文本
  final String leastSelectHintText;

  /// 默认值
  final String defaultValue;

  /// 构建完成后回传控制器
  final Function(DropdownListController controller)? buildCall;

  /// 是否启用
  final bool enable;

  const DropdownList({
    Key? key,
    this.width,
    this.height = 38,
    this.borderColor = const Color(0x60e4e4e4),
    this.buttonBackgroundColors = const [Color(0xff1afa29), Color(0xff1afa29)],
    this.buttonDisableBackgroundColors = const [Color(0x661afa29), Color(0x661afa29)],
    this.buttonIconColor = const Color(0x99333333),
    required this.items,
    this.title,
    this.submit,
    this.submitText,
    this.isPreSelect = true,
    this.isMultiSelect = false,
    required this.selectCall,
    this.hintText = "",
    this.leastSelectHintText = "",
    this.defaultValue = "",
    this.buildCall,
    this.enable = true,
  }) : super(key: key);

  @override
  _DropdownListState createState() => _DropdownListState();
}

class _DropdownListModel extends CusChangeNotifier {
  String displayText = "";
}

class _DropdownListState extends State<DropdownList> with DropdownListController {
  _DropdownListModel model = _DropdownListModel();
  YYDialog? listDialog;
  List<ListItem> _items = [];
  bool _enable = true;

  @override
  void initState() {
    super.initState();
    if (widget.buildCall != null) {
      widget.buildCall!(this);
    }
    _items = widget.items;
    _enable = widget.enable;
    defaultSelectItems();
  }

  void defaultSelectItems() {
    List<ListItem> selectItems = [];
    _items.forEach((element) {
      if (element.value == widget.defaultValue) {
        element.check = true;
        selectItems.add(element);
      } else if (element.check ?? false) {
        selectItems.add(element);
      }
    });
    notifyDisplayText(selectItems, false);
  }

  @override
  Widget build(BuildContext context) {
    return StateUtils.instance.subscribeNotifierWidget<_DropdownListModel>(model, (context, model) {
      return Container(
        width: widget.width,
        height: widget.height,
        decoration: BoxDecoration(
            color: Colors.white,
            borderRadius: BorderRadius.all(Radius.circular(4)),
            border: Border.all(color: widget.borderColor, width: 0.8)),
        child: Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            Expanded(
              child: Padding(
                padding: EdgeInsets.only(left: 6, right: 4),
                child: Text(
                  model.displayText.isEmptyString ? widget.hintText : model.displayText,
                  style: TextStyle(color: Color(0xbe222222), fontSize: 13),
                  maxLines: 1,
                  overflow: TextOverflow.ellipsis,
                ),
              ),
            ),
            buildButton(),
          ],
        ),
      );
    });
  }

  Widget buildButton() {
    return Container(
      child: TextButton(
        child: Icon(
          (listDialog?.isShowing ?? false) ? Icons.keyboard_arrow_up : Icons.keyboard_arrow_down,
          color: widget.buttonIconColor,
        ),
        style: ButtonStyle(
          padding: MaterialStateProperty.all(EdgeInsets.zero),
          shape: MaterialStateProperty.all(RoundedRectangleBorder(borderRadius: BorderRadius.zero)),
          backgroundColor: MaterialStateProperty.all(Colors.transparent),
          visualDensity: VisualDensity.compact,
          minimumSize: MaterialStateProperty.all(Size(50, widget.height + 10)),
        ),
        onPressed: _enable ? showModel : null,
      ),
      width: buttonWidth,
      height: widget.height,
      decoration: BoxDecoration(
        borderRadius: BorderRadius.only(topRight: Radius.circular(4), bottomRight: Radius.circular(4)),
        gradient: LinearGradient(
          colors: _enable ? widget.buttonBackgroundColors : widget.buttonDisableBackgroundColors,
          begin: Alignment.topCenter,
          end: Alignment.bottomCenter,
        ),
      ),
    );
  }

  double get buttonWidth {
    var width = widget.width ?? 0;
    if (width / 5 > 50) {
      return 50;
    } else if (width / 5 < 24) {
      return 24;
    } else {
      return width / 5;
    }
  }

  void showModel() {
    listDialog = Dialogs.instance.showListDialog(context,
        items: _items,
        gravity: Gravity.bottom,
        title: widget.title,
        submit: widget.submit,
        isClickOutsideDismiss: true,
        submitText: widget.submitText,
        isPreSelect: widget.isPreSelect,
        isMultiSelect: widget.isMultiSelect,
        leastSelectHintText: widget.leastSelectHintText, itemPressed: (List<ListItem> selectItems) {
      notifyDisplayText(selectItems, true);
      widget.selectCall(selectItems);
    }, buildCall: (YYDialog dialog) {
      model.notifierLazy();
    }, dismissCall: () {
      Future.delayed(Duration.zero, () {
        model.notifierLazy();
      });
    });
  }

  void notifyDisplayText(List<ListItem> selectItems, bool isNotify) {
    model.displayText = "";
    var count = selectItems.length;
    for (int i = 0; i < count; i++) {
      var item = selectItems[i];
      model.displayText += item.text;
      if ((i + 1) < count) {
        model.displayText += ";";
      }
    }
    model.notifierLazy();
  }

  @override
  notifierLazy(String value) {
    if (widget.isMultiSelect) {
      _items.forEach((element) {
        if (element.value == value) {
          element.check = true;
        }
      });
    } else {
      _items.forEach((element) {
        if (element.value == value) {
          element.check = true;
        } else {
          element.check = false;
        }
      });
    }
  }

  @override
  void enable(bool enable) {
    _enable = enable;
    model.notifierLazy();
  }
}
